==> sk-simple-token-security-1.0.0.tgz: package/index.js <==
const debug = require('debug')('sk-simple-token-security')
const util = require('util')
const jwt = require('jsonwebtoken')
const _ = require('lodash')
const fs = require('fs')

module.exports = function(app) {
  var strategy = {}
  
  var options = getStrategyOptions().configuration
  var secretOrKey = app.config.settings.security.jwtSecretKey;
  
  function setupApp()
  {
    if ( !options )
      options = {}

    debug("options: " + JSON.stringify(options))

    app.use(require('body-parser').urlencoded({ extended: true }));

    app.use(require('cookie-parser')());

    app.get('/loginForm',
            function(req, res) {
              var html = fs.readFileSync(__dirname + '/login.html', {encoding: 'utf8'})

              var insertionIndex = html.indexOf("</form>");
              var result = html.slice(0, insertionIndex);
              var footer = html.slice(insertionIndex);

              if ( req.query.destination ) {
                result += '<input type="hidden" name="destination" value="' + req.query.destination + '">\n';
              }
              
              res.send(result + footer);
            });

    app.post('/login', function(req, res) {
      try
      {
        var name = req.body.username;
        var password = req.body.password;

        debug("username: " + name)
        var user = options.users.find(user => user.username == name )
        if( ! user ){
          res.status(401).send("no such user found");
          return
        }
        if(user.password === password) {
          var payload = {id: user.username};
          var expiration = options.jwtExpiration || '1h'
          debug('jwt expiration: ' + expiration)
          var token = jwt.sign(payload, secretOrKey, {expiresIn: expiration} );

          var requestType = req.get('Content-Type');
          if ( requestType == 'application/json' ) {
            res.json({token: token});
          } else {
            res.cookie('JAUTHENTICATION', token, { httpOnly: true });
            res.redirect(req.body.destination ? req.body.destination : '/')
          }

        } else {
          res.status(401).send("passwords did not match");
        }
      }
      catch ( err ) {
        console.log(err)
      }
    });
    

    var do_redir = http_authorize(true)

    app.use('/', function(req, res, next) {
      debug("do_redir: " + req.path)
      if ( req.path == '/' )
      {
        do_redir(req, res, next)
      }
      else
      {
        next()
      }
    })
    app.use('/apps', http_authorize(true))
    app.use('/appstore', http_authorize(true))
    app.use('/plugins', http_authorize(true))
    app.use('/restart', http_authorize(false))

    app.get('/logout', function(req, res) {
      res.clearCookie('JAUTHENTICATION')
      res.redirect('/');
    });

    
    function adminAuthenticationMiddleware(redirect) {  
      return function (req, res, next) {
        debug("isAuthenticated: " + req.isAuthenticated)
        if (req.isAuthenticated) {
          if ( req.user.type == 'admin' ) {
            return next()
          }
        }
        if ( redirect ) {
          res.redirect('/loginForm?destination='+req.originalUrl)
        } else {
          res.status(401)
          res.send("Permission Denied")
        }
      }
    }

    function readOnlyAuthenticationMiddleware(redirect) {  
      return function (req, res, next) {
        debug("isAuthenticated: " + req.isAuthenticated)
        if (req.isAuthenticated) {
          if ( ['admin', 'readonly', 'readwrite'].find(type => req.user.type == type )) {
            return next()
          }
        }
        if ( redirect ) {
          res.redirect('/loginForm?destination='+req.originalUrl)
        } else {
          res.status(401)
          res.send("Permission Denied")
        }
      }
    }

    app.use('/restart', adminAuthenticationMiddleware(false))
    app.use('/plugins', adminAuthenticationMiddleware(true))
    app.use('/appstore', adminAuthenticationMiddleware(true))
    //app.use('/plugins/*/config', plugin.adminAuthenticationMiddleware(false))

    var no_redir = http_authorize(false)
    app.use('/signalk/v1/', function(req, res, next) {
      no_redir(req, res, next)
    });
  }
    
  strategy.shouldAllowWrite = function(spark) {
    return spark.user && (spark.user.type === 'admin' || spark.user.type === 'readwrite');
  }

  strategy.verifyWS = function(spark) {
    if ( ! spark.lastTokenVerify )
    {
      spark.lastTokenVerify = new Date()
      return
    }
    var now = new Date()
    if ( now - spark.lastTokenVerify > 60*1000 )
    {
      debug("verify token")
      spark.lastTokenVerify = now
      strategy.authorizeWS(spark)
    }
  }
  
  strategy.authorizeWS = function(req)
  {
    var token = req.query.token
    , error
    , payload;

    if ( ! token )
      {
        var header = req.headers.authorization
        if ( header && header.startsWith('JWT ') ) {
          token = header.substring('JWT '.length)
        }
      }

    if (!token) {
      if ( options.allow_readonly )
      {
        req.user = {'id': 'AUTO', 'type': 'readonly' };
        return
      }
      else
      {
        error = new Error('Missing access token');
        debug(error.message);
        throw error
      }
    }

    //
    // `jwt-simple` throws errors if something goes wrong when decoding the JWT.
    //
    payload = jwt.decode(token, secretOrKey);

    if ( !payload ) {
      error = new Error('Invalid access token');
      debug(error.message);
      throw error;
    }

    //
    // At this point we have decoded and verified the token. Check if it is
    // expired.
    //
    if (Date.now()/1000 > payload.exp) {
      error = new Error('Expired access token');
      debug(error.message);
      throw error;
    }

    //
    // Check if the user is still present and allowed in our db. You could tweak
    // this to invalidate a token.
    //
    var user = options.users.find(user => user.username == payload.id )
    if ( !user ) {
      error = new Error('Invalid access token');
      debug(error.message);
      throw error;
    }

    req.user = user
  }

  function http_authorize(redirect) {
    //debug('http_authorize: ' + redirect)
    return function (req, res, next) {
      var token = req.cookies.JAUTHENTICATION;

      debug("http_authorize: " + req.path)

      if ( ! token ) {
        var header = req.headers.authorization
        if ( header && header.startsWith('JWT ') ) {
          token = header.substring('JWT '.length)
        }
      }

      if ( token ) {
        jwt.verify(token, secretOrKey, function(err, decoded) {
          debug("verify")
          if ( !err ) {
            var user = options.users.find(user => user.username == decoded.id)
            if ( user )
            {
              debug("authorized")
              req.user = decoded;
              req.user.type = user.type
              req.isAuthenticated = true
              req.userLoggedIn = true
              next()
              return;
            }
            else
            {
              debug("unknown user: " + decoded.id)
            }
          }
          else
          {
            debug("bad token: " + req.path)
          }
          req.isAuthenticated = false
          
          if ( redirect ) {
            res.redirect('/loginForm?destination='+req.originalUrl)
          } else {
            res.status(401).send("bad auth token");
          }
        });
      }
      else
      {
        debug("no token")

        if ( options.allow_readonly )
        {
          req.user = {'id': 'AUTO', 'type': 'readonly' };
          req.isAuthenticated = true
          next()
        }
        else
        {
          req.isAuthenticated = false
          
          if ( redirect ) {
            res.redirect('/loginForm?destination='+req.originalUrl)
          } else {
            res.status(401).send("bad auth token");
          }
        }
      }
    }
  }

  setupApp()

  return strategy
}

function pathForStrategyConfig() {
  return "./plugin-config-data/sk-simple-token-security-config.json"
}

function getStrategyOptions() {
  try {
    const optionsAsString = fs.readFileSync(pathForStrategyConfig(), 'utf8');
    try {
      const options = JSON.parse(optionsAsString)
      return options
    } catch(e) {
      console.error("Could not parse JSON options:" + optionsAsString);
      return {}
    }
  } catch(e) {
    debug("Could not find options for security, returning empty options")
    return {}
  }
}

==> sk-simple-token-security-1.2.0.tgz: package/index.js <==
const debug = require('debug')('sk-simple-token-security')
const util = require('util')
const jwt = require('jsonwebtoken')
const _ = require('lodash')
const fs = require('fs')
const path = require('path');

module.exports = function(app) {
  var strategy = {}
  
  var options
  var secretOrKey = app.config.settings.security.jwtSecretKey;

  function getOptions()
  {
    if ( !options ) {
      options = getStrategyOptions(app)

      fs.watchFile(pathForStrategyConfig(app), (curr, prev) => {
        debug("options changed, reloading...")
        options = getStrategyOptions(app)
      });
      
    }
    return options
  }

  function getConfiguration()
  {
    return getOptions().configuration
  }

  function getIsEnabled() {
    var options = getOptions();
    return typeof options.enabled !== 'undefined' && options.enabled;
  }
  
  function setupApp()
  {
    debug("configuration: " + JSON.stringify(getConfiguration()))

    app.use(require('body-parser').urlencoded({ extended: true }));

    app.use(require('cookie-parser')());

    app.get('/loginForm',
            function(req, res) {
              var html = fs.readFileSync(__dirname + '/login.html', {encoding: 'utf8'})

              var insertionIndex = html.indexOf("</form>");
              var result = html.slice(0, insertionIndex);
              var footer = html.slice(insertionIndex);

              if ( req.query.destination ) {
                result += '<input type="hidden" name="destination" value="' + req.query.destination + '">\n';
              }
              
              res.send(result + footer);
            });

    app.post('/login', function(req, res) {
      try
      {
        var name = req.body.username;
        var password = req.body.password;

        debug("username: " + name)
        var configuration = getConfiguration()
        var user = configuration.users.find(user => user.username == name )
        if( ! user ){
          res.status(401).send("no such user found");
          return
        }
        if(user.password === password) {
          var payload = {id: user.username};
          var expiration = configuration.jwtExpiration || '1h'
          debug('jwt expiration: ' + expiration)
          var token = jwt.sign(payload, secretOrKey, {expiresIn: expiration} );

          var requestType = req.get('Content-Type');
          if ( requestType == 'application/json' ) {
            res.json({token: token});
          } else {
            res.cookie('JAUTHENTICATION', token, { httpOnly: true });
            res.redirect(req.body.destination ? req.body.destination : '/')
          }

        } else {
          res.status(401).send("passwords did not match");
        }
      }
      catch ( err ) {
        console.log(err)
      }
    });
    

    var do_redir = http_authorize(true)

    app.use('/', function(req, res, next) {
      debug("do_redir: " + req.path)
      if ( req.path == '/' )
      {
        do_redir(req, res, next)
      }
      else
      {
        next()
      }
    })
    app.use('/apps', http_authorize(true))
    app.use('/appstore', http_authorize(true))
    app.use('/plugins', http_authorize(true))
    app.use('/restart', http_authorize(false))

    app.get('/logout', function(req, res) {
      res.clearCookie('JAUTHENTICATION')
      res.redirect('/');
    });

    
    function adminAuthenticationMiddleware(redirect) {  
      return function (req, res, next) {

        if ( !getIsEnabled() ) {
          return next()
        }
        
        debug("isAuthenticated: " + req.isAuthenticated)
        if (req.isAuthenticated) {
          if ( req.user.type == 'admin' ) {
            return next()
          }
        }
        if ( redirect ) {
          res.redirect('/loginForm?destination='+req.originalUrl)
        } else {
          res.status(401)
          res.send("Permission Denied")
        }
      }
    }

    function readOnlyAuthenticationMiddleware(redirect) {  
      return function (req, res, next) {
        
        if ( !getIsEnabled() ) {
          return next()
        }
        debug("isAuthenticated: " + req.isAuthenticated)
        if (req.isAuthenticated) {
          if ( ['admin', 'readonly', 'readwrite'].find(type => req.user.type == type )) {
            return next()
          }
        }
        if ( redirect ) {
          res.redirect('/loginForm?destination='+req.originalUrl)
        } else {
          res.status(401)
          res.send("Permission Denied")
        }
      }
    }

    app.use('/restart', adminAuthenticationMiddleware(false))
    app.use('/plugins', adminAuthenticationMiddleware(true))
    app.use('/appstore', adminAuthenticationMiddleware(true))
    //app.use('/plugins/*/config', plugin.adminAuthenticationMiddleware(false))

    var no_redir = http_authorize(false)
    app.use('/signalk/v1/', function(req, res, next) {
      no_redir(req, res, next)
    });
  }
    
  strategy.shouldAllowWrite = function(spark) {
    return getIsEnabled() == flase || (spark.user && (spark.user.type === 'admin' || spark.user.type === 'readwrite'));
  }

  strategy.verifyWS = function(spark) {
    if ( ! spark.lastTokenVerify )
    {
      spark.lastTokenVerify = new Date()
      return
    }
    
    if ( !getIsEnabled() ) {
      return
    }
    
    var now = new Date()
    if ( now - spark.lastTokenVerify > 60*1000 )
    {
      debug("verify token")
      spark.lastTokenVerify = now
      strategy.authorizeWS(spark)
    }
  }
  
  strategy.authorizeWS = function(req)
  {
    var token = req.query.token
    , error
    , payload;

    if ( !getIsEnabled() ) {
      return
    }

    var configuration = getConfiguration()

    if ( ! token )
      {
        var header = req.headers.authorization
        if ( header && header.startsWith('JWT ') ) {
          token = header.substring('JWT '.length)
        }
      }

    if (!token) {
      if ( configuration.allow_readonly )
      {
        req.user = {'id': 'AUTO', 'type': 'readonly' };
        return
      }
      else
      {
        error = new Error('Missing access token');
        debug(error.message);
        throw error
      }
    }

    //
    // `jwt-simple` throws errors if something goes wrong when decoding the JWT.
    //
    payload = jwt.decode(token, secretOrKey);

    if ( !payload ) {
      error = new Error('Invalid access token');
      debug(error.message);
      throw error;
    }

    //
    // At this point we have decoded and verified the token. Check if it is
    // expired.
    //
    if (Date.now()/1000 > payload.exp) {
      error = new Error('Expired access token');
      debug(error.message);
      throw error;
    }

    //
    // Check if the user is still present and allowed in our db. You could tweak
    // this to invalidate a token.
    //
    var user = configuration.users.find(user => user.username == payload.id )
    if ( !user ) {
      error = new Error('Invalid access token');
      debug(error.message);
      throw error;
    }

    req.user = user
  }

  function http_authorize(redirect) {
    //debug('http_authorize: ' + redirect)
    return function (req, res, next) {
      var token = req.cookies.JAUTHENTICATION;

      debug("http_authorize: " + req.path)

      if ( !getIsEnabled() ) {
        return next()
      }

      var configuration = getConfiguration()

      if ( ! token ) {
        var header = req.headers.authorization
        if ( header && header.startsWith('JWT ') ) {
          token = header.substring('JWT '.length)
        }
      }

      if ( token ) {
        jwt.verify(token, secretOrKey, function(err, decoded) {
          debug("verify")
          if ( !err ) {
            var user = configuration.users.find(user => user.username == decoded.id)
            if ( user )
            {
              debug("authorized")
              req.user = decoded;
              req.user.type = user.type
              req.isAuthenticated = true
              req.userLoggedIn = true
              next()
              return;
            }
            else
            {
              debug("unknown user: " + decoded.id)
            }
          }
          else
          {
            debug("bad token: " + req.path)
          }
          req.isAuthenticated = false
          
          if ( redirect ) {
            res.redirect('/loginForm?destination='+req.originalUrl)
          } else {
            res.status(401).send("bad auth token");
          }
        });
      }
      else
      {
        debug("no token")

        if ( configuration.allow_readonly )
        {
          req.user = {'id': 'AUTO', 'type': 'readonly' };
          req.isAuthenticated = true
          return next()
        }
        else
        {
          req.isAuthenticated = false
          
          if ( redirect ) {
            res.redirect('/loginForm?destination='+req.originalUrl)
          } else {
            res.status(401).send("bad auth token");
          }
        }
      }
    }
  }

  setupApp()

  return strategy
}

function pathForStrategyConfig(app) {
  var dir =  app.config.configPath ? app.config.configPath : "./";

  var res = path.join(dir, "plugin-config-data", "sk-simple-token-security-config.json")

  debug("loading config from " + res);

  return res
}

function getStrategyOptions(app) {
  try {
    const optionsAsString = fs.readFileSync(pathForStrategyConfig(app), 'utf8');
    try {
      const options = JSON.parse(optionsAsString)
      return options
    } catch(e) {
      console.error("Could not parse JSON options:" + optionsAsString);
      return {}
    }
  } catch(e) {
    debug("Could not find options for security, returning empty options")
    return {}
  }
}
